@@ -34,7 +34,6 @@ G_DEFINE_ABSTRACT_TYPE (CEPage, ce_page, G_TYPE_OBJECT)
 enum {
 	PROP_0,
 	PROP_CONNECTION,
-	PROP_INITIALIZED,
 	PROP_PARENT_WINDOW,
 
 	LAST_PROP
@@ -43,6 +42,7 @@ enum {
 enum {
 	CHANGED,
 	INITIALIZED,
+	NEW_EDITOR,
 
 	LAST_SIGNAL
 };
@@ -382,15 +382,15 @@ gboolean
 ce_page_interface_name_valid (const char *iface, const char *property_name, GError **error)
 {
 	if (iface && *iface) {
-		if (!nm_utils_iface_valid_name (iface)) {
+		if (!nm_utils_is_valid_iface_name (iface, error)) {
 			if (property_name) {
-				g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC,
-				             _("invalid interface-name for %s (%s)"),
-				             property_name, iface);
+				g_prefix_error (error,
+				                _("invalid interface-name for %s (%s): "),
+				                property_name, iface);
 			} else {
-				g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC,
-				             _("invalid interface-name (%s)"),
-				             iface);
+				g_prefix_error (error,
+				                _("invalid interface-name (%s): "),
+				                iface);
 			}
 			return FALSE;
 		}
@@ -556,9 +556,9 @@ gboolean
 ce_page_device_entry_get (GtkEntry *entry, int type, gboolean check_ifname,
                           char **ifname, char **mac, const char *device_name, GError **error)
 {
-	char *first, *second;
+	gs_free char *first = NULL;
+	gs_free char *second = NULL;
 	const char *ifname_tmp = NULL, *mac_tmp = NULL;
-	gboolean valid = TRUE;
 	const char *str;
 
 	g_return_val_if_fail (entry != NULL, FALSE);
@@ -566,29 +566,36 @@ ce_page_device_entry_get (GtkEntry *entry, int type, gboolean check_ifname,
 
 	str = gtk_entry_get_text (entry);
 
-	valid = _device_entry_parse (str, &first, &second);
+	if (!_device_entry_parse (str, &first, &second)) {
+		g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC,
+		                     _("can’t parse device name"));
+		goto invalid;
+	}
 
 	if (first) {
 		if (nm_utils_hwaddr_valid (first, nm_utils_hwaddr_len (type)))
 			mac_tmp = first;
-		else if (!check_ifname || nm_utils_iface_valid_name (first))
+		else if (!check_ifname || nm_utils_is_valid_iface_name (first, error))
 			ifname_tmp = first;
 		else
-			valid = FALSE;
+			goto invalid;
 	}
 	if (second) {
 		if (nm_utils_hwaddr_valid (second, nm_utils_hwaddr_len (type))) {
-			if (!mac_tmp)
+			if (!mac_tmp) {
 				mac_tmp = second;
-			else
-				valid = FALSE;
-		} else if (!check_ifname || nm_utils_iface_valid_name (second)) {
+			} else {
+				g_set_error_literal (error, NMA_ERROR, NMA_ERROR_GENERIC,
+				                     _("invalid hardware address"));
+				goto invalid;
+			}
+		} else if (!check_ifname || nm_utils_is_valid_iface_name (second, error)) {
 			if (!ifname_tmp)
 				ifname_tmp = second;
 			else
-				valid = FALSE;
+				goto invalid;
 		} else
-			valid = FALSE;
+			goto invalid;
 	}
 
 	if (ifname)
@@ -596,17 +603,22 @@ ce_page_device_entry_get (GtkEntry *entry, int type, gboolean check_ifname,
 	if (mac)
 		*mac = g_strdup (mac_tmp);
 
-	g_free (first);
-	g_free (second);
+	return TRUE;
 
-	if (!valid) {
+invalid:
+	if (error) {
+		g_prefix_error (error,
+		                _("invalid %s (%s): "),
+		                device_name ? device_name : _("device"),
+		                str);
+	} else {
 		g_set_error (error, NMA_ERROR, NMA_ERROR_GENERIC,
-		             _("invalid %s (%s)"),
-		             device_name ? device_name : _("device"),
-		             str);
+		             _("invalid %s (%s) "),
+		            device_name ? device_name : _("device"),
+		            str);
 	}
 
-	return valid;
+	return FALSE;
 }
 
 char *
@@ -648,13 +660,6 @@ ce_page_get_next_available_name (const GPtrArray *connections, const char *forma
 	return cname;
 }
 
-static void
-emit_initialized (CEPage *self, GError *error)
-{
-	self->initialized = TRUE;
-	g_signal_emit (self, signals[INITIALIZED], 0, error);
-}
-
 void
 ce_page_complete_init (CEPage *self,
                        const char *setting_name,
@@ -664,27 +669,29 @@ ce_page_complete_init (CEPage *self,
 	GError *update_error = NULL;
 	GVariant *setting_dict;
 	char *dbus_err;
-	gboolean ignore_error = FALSE;
 
 	g_return_if_fail (self != NULL);
 	g_return_if_fail (CE_IS_PAGE (self));
 
 	if (error) {
+		/* Ignore missing settings errors */
 		dbus_err = g_dbus_error_get_remote_error (error);
-		ignore_error =    !g_strcmp0 (dbus_err, "org.freedesktop.NetworkManager.Settings.InvalidSetting")
-		               || !g_strcmp0 (dbus_err, "org.freedesktop.NetworkManager.Settings.Connection.SettingNotFound")
-		               || !g_strcmp0 (dbus_err, "org.freedesktop.NetworkManager.AgentManager.NoSecrets");
+		if (   g_strcmp0 (dbus_err, "org.freedesktop.NetworkManager.Settings.InvalidSetting") == 0
+		    || g_strcmp0 (dbus_err, "org.freedesktop.NetworkManager.Settings.Connection.SettingNotFound") == 0
+		    || g_strcmp0 (dbus_err, "org.freedesktop.NetworkManager.AgentManager.NoSecrets") == 0)
+			g_clear_error (&error);
 		g_free (dbus_err);
 	}
 
-	/* Ignore missing settings errors */
-	if (error && !ignore_error) {
-		emit_initialized (self, error);
-		return;
-	} else if (!setting_name || !secrets || g_variant_n_children (secrets) == 0) {
+	if (error) {
+		g_warning ("Couldn't fetch secrets: %s", error->message);
+		g_error_free (error);
+		goto out;
+	}
+
+	if (!setting_name || !secrets || g_variant_n_children (secrets) == 0) {
 		/* Success, no secrets */
-		emit_initialized (self, NULL);
-		return;
+		goto out;
 	}
 
 	g_assert (setting_name);
@@ -693,28 +700,22 @@ ce_page_complete_init (CEPage *self,
 	setting_dict = g_variant_lookup_value (secrets, setting_name, NM_VARIANT_TYPE_SETTING);
 	if (!setting_dict) {
 		/* Success, no secrets */
-		emit_initialized (self, NULL);
-		return;
+		goto out;
 	}
 	g_variant_unref (setting_dict);
 
 	/* Update the connection with the new secrets */
-	if (nm_connection_update_secrets (self->connection,
+	if (!nm_connection_update_secrets (self->connection,
 	                                  setting_name,
 	                                  secrets,
 	                                  &update_error)) {
-		/* Success */
-		emit_initialized (self, NULL);
-		return;
+		g_warning ("Couldn't update the secrets: %s", update_error->message);
+		g_error_free (update_error);
+		goto out;
 	}
 
-	if (!update_error) {
-		g_set_error_literal (&update_error, NMA_ERROR, NMA_ERROR_GENERIC,
-		                     _("Failed to update connection secrets due to an unknown error."));
-	}
-
-	emit_initialized (self, update_error);
-	g_clear_error (&update_error);
+out:
+	g_signal_emit (self, signals[INITIALIZED], 0, NULL);
 }
 
 static void
@@ -761,14 +762,6 @@ ce_page_get_title (CEPage *self)
 	return self->title;
 }
 
-gboolean
-ce_page_get_initialized (CEPage *self)
-{
-	g_return_val_if_fail (CE_IS_PAGE (self), FALSE);
-
-	return self->initialized;
-}
-
 void
 ce_page_changed (CEPage *self)
 {
@@ -777,6 +770,25 @@ ce_page_changed (CEPage *self)
 	g_signal_emit (self, signals[CHANGED], 0);
 }
 
+NMConnectionEditor *
+ce_page_new_editor (CEPage *self,
+                    GtkWindow *parent_window,
+                    NMConnection *connection)
+{
+	NMConnectionEditor *editor;
+
+	g_return_val_if_fail (CE_IS_PAGE (self), NULL);
+
+	editor = nm_connection_editor_new (parent_window,
+	                                   connection,
+	                                   self->client);
+	if (!editor)
+		return NULL;
+
+	g_signal_emit (self, signals[NEW_EDITOR], 0, editor);
+	return editor;
+}
+
 static void
 get_property (GObject *object, guint prop_id,
               GValue *value, GParamSpec *pspec)
@@ -787,9 +799,6 @@ get_property (GObject *object, guint prop_id,
 	case PROP_CONNECTION:
 		g_value_set_object (value, self->connection);
 		break;
-	case PROP_INITIALIZED:
-		g_value_set_boolean (value, self->initialized);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -837,14 +846,6 @@ ce_page_class_init (CEPageClass *page_class)
 		                      NM_TYPE_CONNECTION,
 		                      G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 
-	g_object_class_install_property
-		(object_class, PROP_INITIALIZED,
-		 g_param_spec_boolean (CE_PAGE_INITIALIZED,
-		                       "Initialized",
-		                       "Initialized",
-		                       FALSE,
-		                       G_PARAM_READABLE));
-
 	g_object_class_install_property
 		(object_class, PROP_PARENT_WINDOW,
 		 g_param_spec_pointer (CE_PAGE_PARENT_WINDOW,
@@ -857,16 +858,21 @@ ce_page_class_init (CEPageClass *page_class)
 		g_signal_new ("changed",
 	                      G_OBJECT_CLASS_TYPE (object_class),
 	                      G_SIGNAL_RUN_FIRST,
-	                      G_STRUCT_OFFSET (CEPageClass, changed),
-	                      NULL, NULL, NULL,
+	                      0, NULL, NULL, NULL,
 	                      G_TYPE_NONE, 0);
 
 	signals[INITIALIZED] = 
-		g_signal_new ("initialized",
+		g_signal_new (CE_PAGE_INITIALIZED,
 	                      G_OBJECT_CLASS_TYPE (object_class),
 	                      G_SIGNAL_RUN_FIRST,
-	                      G_STRUCT_OFFSET (CEPageClass, initialized),
-	                      NULL, NULL, NULL,
+	                      0, NULL, NULL, NULL,
+	                      G_TYPE_NONE, 1, G_TYPE_POINTER);
+
+	signals[NEW_EDITOR] =
+		g_signal_new (CE_PAGE_NEW_EDITOR,
+	                      G_OBJECT_CLASS_TYPE (object_class),
+	                      G_SIGNAL_RUN_FIRST,
+	                      0, NULL, NULL, NULL,
 	                      G_TYPE_NONE, 1, G_TYPE_POINTER);
 }
 
