@@ -70,13 +70,10 @@ populate_ui (CEPageBridgePort *self)
 }
 
 static void
-finish_setup (CEPageBridgePort *self, gpointer unused, GError *error, gpointer user_data)
+finish_setup (CEPageBridgePort *self, gpointer user_data)
 {
 	CEPageBridgePortPrivate *priv = CE_PAGE_BRIDGE_PORT_GET_PRIVATE (self);
 
-	if (error)
-		return;
-
 	populate_ui (self);
 
 	g_signal_connect (priv->priority, "value-changed", G_CALLBACK (stuff_changed), self);
@@ -120,7 +117,7 @@ ce_page_bridge_port_new (NMConnectionEditor *editor,
 		nm_connection_add_setting (connection, NM_SETTING (priv->setting));
 	}
 
-	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
+	g_signal_connect (self, CE_PAGE_INITIALIZED, G_CALLBACK (finish_setup), NULL);
 
 	return CE_PAGE (self);
 }
