@@ -46,17 +46,7 @@ typedef struct {
 
 } CEPageMasterPrivate;
 
-enum {
-	CREATE_CONNECTION,
-	CONNECTION_ADDED,
-	CONNECTION_REMOVED,
-
-	LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL] = { 0 };
-
-static void finish_setup (CEPageMaster *self, gpointer unused, GError *error, gpointer user_data);
+static void finish_setup (CEPageMaster *self, gpointer user_data);
 
 enum {
 	COL_CONNECTION,
@@ -84,7 +74,7 @@ constructed (GObject *object)
 {
 	CEPageMaster *self = CE_PAGE_MASTER (object);
 
-	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
+	g_signal_connect (self, CE_PAGE_INITIALIZED, G_CALLBACK (finish_setup), NULL);
 
 	G_OBJECT_CLASS (ce_page_master_parent_class)->constructed (object);
 }
@@ -158,8 +148,6 @@ connection_removed (NMClient *client,
 
 	gtk_list_store_remove (GTK_LIST_STORE (priv->connections_model), &iter);
 	ce_page_changed (CE_PAGE (self));
-
-	g_signal_emit (self, signals[CONNECTION_REMOVED], 0, connection);
 }
 
 static void
@@ -312,8 +300,6 @@ connection_added (NMClient *client,
 	                  G_CALLBACK (connection_removed), self);
 	g_signal_connect (connection, NM_CONNECTION_CHANGED,
 	                  G_CALLBACK (connection_changed), self);
-
-	g_signal_emit (self, signals[CONNECTION_ADDED], 0, connection);
 }
 
 static void
@@ -393,15 +379,11 @@ add_connection (FUNC_TAG_NEW_CONNECTION_RESULT_IMPL,
 	              NULL);
 	g_free (name);
 
-	g_signal_emit (self, signals[CREATE_CONNECTION], 0, connection);
-
-	editor = nm_connection_editor_new (priv->toplevel,
-	                                   connection,
-	                                   CE_PAGE (self)->client);
+	editor = ce_page_new_editor (CE_PAGE (self), priv->toplevel, connection);
 	if (!editor)
 		return;
 
-	g_signal_connect (editor, "done", G_CALLBACK (add_response_cb), self);
+	g_signal_connect (editor, NM_CONNECTION_EDITOR_DONE, G_CALLBACK (add_response_cb), self);
 	nm_connection_editor_run (editor);
 }
 
@@ -460,13 +442,11 @@ edit_clicked (GtkButton *button, gpointer user_data)
 		return;
 	}
 
-	editor = nm_connection_editor_new (priv->toplevel,
-	                                   NM_CONNECTION (connection),
-	                                   CE_PAGE (self)->client);
+	editor = ce_page_new_editor (CE_PAGE (self), priv->toplevel, NM_CONNECTION (connection));
 	if (!editor)
 		return;
 
-	g_signal_connect (editor, "done", G_CALLBACK (edit_done_cb), self);
+	g_signal_connect (editor, NM_CONNECTION_EDITOR_DONE, G_CALLBACK (edit_done_cb), self);
 	nm_connection_editor_run (editor);
 }
 
@@ -531,16 +511,13 @@ populate_ui (CEPageMaster *self)
 }
 
 static void
-finish_setup (CEPageMaster *self, gpointer unused, GError *error, gpointer user_data)
+finish_setup (CEPageMaster *self, gpointer user_data)
 {
 	CEPageMasterPrivate *priv = CE_PAGE_MASTER_GET_PRIVATE (self);
 	GtkTreeSelection *selection;
 	GtkBuilder *builder;
 	NMSettingConnection *s_con;
 
-	if (error)
-		return;
-
 	builder = CE_PAGE (self)->builder;
 
 	priv->interface_name = GTK_ENTRY (gtk_builder_get_object (builder, "master_interface"));
@@ -675,34 +652,6 @@ ce_page_master_class_init (CEPageMasterClass *master_class)
 
 	parent_class->ce_page_validate_v = ce_page_validate_v;
 	parent_class->last_update = last_update;
-
-	/* Signals */
-	signals[CREATE_CONNECTION] = 
-		g_signal_new ("create-connection",
-		              G_OBJECT_CLASS_TYPE (object_class),
-		              G_SIGNAL_RUN_FIRST,
-		              G_STRUCT_OFFSET (CEPageMasterClass, create_connection),
-		              NULL, NULL, NULL,
-		              G_TYPE_NONE, 1,
-		              NM_TYPE_CONNECTION);
-
-	signals[CONNECTION_ADDED] = 
-		g_signal_new ("connection-added",
-		              G_OBJECT_CLASS_TYPE (object_class),
-		              G_SIGNAL_RUN_FIRST,
-		              G_STRUCT_OFFSET (CEPageMasterClass, connection_added),
-		              NULL, NULL, NULL,
-		              G_TYPE_NONE, 1,
-		              NM_TYPE_CONNECTION);
-
-	signals[CONNECTION_REMOVED] = 
-		g_signal_new ("connection-removed",
-		              G_OBJECT_CLASS_TYPE (object_class),
-		              G_SIGNAL_RUN_FIRST,
-		              G_STRUCT_OFFSET (CEPageMasterClass, connection_removed),
-		              NULL, NULL, NULL,
-		              G_TYPE_NONE, 1,
-		              NM_TYPE_CONNECTION);
 }
 
 gboolean
