@@ -99,7 +99,8 @@ update_secrets (WirelessSecurity *parent, NMConnection *connection)
 WirelessSecurityWPAEAP *
 ws_wpa_eap_new (NMConnection *connection,
                 gboolean is_editor,
-                gboolean secrets_only)
+                gboolean secrets_only,
+                const char *const*secrets_hints)
 {
 	WirelessSecurity *parent;
 	GtkWidget *widget;
@@ -125,7 +126,8 @@ ws_wpa_eap_new (NMConnection *connection,
 	                                    (GCallback) auth_combo_changed_cb,
 	                                    connection,
 	                                    is_editor,
-	                                    secrets_only);
+	                                    secrets_only,
+	                                    secrets_hints);
 	auth_combo_changed_cb (widget, parent);
 
 	return (WirelessSecurityWPAEAP *) parent;
